@@ -182,6 +182,7 @@ static gchar *contributors[] = {
 	"Paolo Maggi",
 	"Sam Magnuson",
 	"Ian Main",
+        "Jordi Mallach",
 	"Mandrake",
 	"Daniel Manrique",
 	"Kjartan Maraas",
@@ -315,6 +316,7 @@ static gchar *contributors[] = {
 	"Vincent Untz",
 	N_("Manish Vachharajani"),
 	N_("Neil Vachharajani"),
+	"Thomas Vander Stichele",
 	"Daniel Veillard",
 	"Vendu",
 	"Andrew Veliath",
